% Este ejemplo permite visualizar un frame de un video en formato YUV 4:2:0
% El bloque procesado es de 16x16 con una funcin coste SAD
% Algoritmo full-search
clear all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Area de busqueda (16+2*searchArea)*(16+2*searchArea)
searchArea=32;

% Seleccin de QP
QP=15;

% Caractersticas del video a procesar
high=288;
width=352;
numTotalFrames=21;
nameVideo='Vectra@21@352x288.yuv';
numReferenceFrame=1;
numCurrentFrame=3;

% Se leen los ficheros current y reference video
[Yvid, Uvid, Vvid]=leerVideoYUV_420(nameVideo, high, width, numTotalFrames);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Se selecciona el origen del bloque actual a procesar
posX=16*10;
posY=16*9;

% Seleccion MB 16x16 de current frame
% +1 porque el ndice en matrices de matlab empieza en 1, no en 0
YC=squeeze( Yvid(posX+1:posX+16,posY+1:posY+16,numCurrentFrame ));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Yr=squeeze(Yvid(posX-searchArea+1:posX+searchArea+16,posY-searchArea+1:posY+searchArea+16,numReferenceFrame ));
Ur=squeeze(Uvid((posX-searchArea)/2+1:(posX+searchArea+16)/2,(posY-searchArea)/2+1:(posY+16+searchArea)/2,numReferenceFrame ));
Vr=squeeze(Vvid((posX-searchArea)/2+1:(posX+searchArea+16)/2,(posY-searchArea)/2+1:(posY+16+searchArea)/2,numReferenceFrame ));
subplot(1,2,1), imshow(ycbcr2rgb(cat(3,Yr,imresize(Ur,2), imresize(Vr,2)))),...
    title(sprintf('IMAGEN REFERENCIA %dx%d',2*searchArea+16,2*searchArea+16));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Yc=squeeze(Yvid(posX+1:posX+16,posY+1:posY+16,numCurrentFrame ));
Uc=squeeze(Uvid(posX/2+1:posX/2+8,posY/2+1:posY/2+8,numCurrentFrame ));
Vc=squeeze(Vvid(posX/2+1:posX/2+8,posY/2+1:posY/2+8,numCurrentFrame ));
subplot(1,2,2), imshow(ycbcr2rgb(cat(3,Yc,imresize(Uc,2), imresize(Vc,2)))),title('IMAGEN CURRENT 16x16');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Se calcula los diferentes costes SAD y el vector de movimiento en cada caso
minSAD=65535;
% Se recorre todos los bloques 16x16 dentro del rea de bsqueda
for dx=-searchArea:searchArea
    for dy=-searchArea:searchArea
        % Se selecciona un bloque 16x16 en el area de bsqueda
        YRsel(1:16,1:16)=squeeze( Yvid(posX+dx+1:posX+dx+16,posY+dy+1:posY+dy+16, numReferenceFrame));
        
        % Se llama a la funcin coste SAD
        SAD = costFuncSAD(YRsel, YC);
        
        % Se comprueba el minimo
        if (SAD < minSAD)
            minSAD=SAD;
            minMVx=dx;
            minMVy=dy;
        end
    end
end

% Resultado del mejor estimacin de movimiento
fprintf(1,'\n BEST MATCH: REFERENCE FRAME=%2d CURRENT FRAME=%2d -> SAD=%5d MVx=%2d MVy=%2d\n',...
    numReferenceFrame, numCurrentFrame, minSAD,minMVx, minMVy);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compensacion
% Best match in la frame de refencia;
Ybest=squeeze ( Yvid(posX+minMVx+1:posX+minMVx+16,posY+minMVy+1:posY+minMVy+16,numReferenceFrame));
%

% Solo LUMAS
interIntra=0;

fprintf(1,'\n IMAGEN RESIDUAL:\n');
Ycomp=floor(double(Ybest)-double(YC));
disp(Ycomp);

% Calculo de las transformadas y cuantizaciones
for lazoX=0:3
    for lazoY=0:3
        % Transformada directa
        W(1+lazoX*4:4+lazoX*4, 1+lazoY*4:4+lazoY*4)=DCT_forward(Ycomp(1+lazoX*4:4+lazoX*4, 1+lazoY*4:4+lazoY*4));
        % Cuantizacin directa
        Z(1+lazoX*4:4+lazoX*4, 1+lazoY*4:4+lazoY*4)=Quantization_forward( W(1+lazoX*4:4+lazoX*4, 1+lazoY*4:4+lazoY*4), QP, interIntra);
        % Cuantizacin inversa u escalado
        WI(1+lazoX*4:4+lazoX*4, 1+lazoY*4:4+lazoY*4)=Quantization_inverse (Z(1+lazoX*4:4+lazoX*4, 1+lazoY*4:4+lazoY*4), QP);
        % Transformada inversa
        xr(1+lazoX*4:4+lazoX*4, 1+lazoY*4:4+lazoY*4)=DCT_inverse (WI(1+lazoX*4:4+lazoX*4, 1+lazoY*4:4+lazoY*4))  ;
    end
end

fprintf(1,'\n TRANSFORMADA DIRECTA:\n');
disp(W);
fprintf(1,'\n CUANTIZACIN DIRECTA:\n');
disp(Z);
fprintf(1,'\n CUANTIZACIN INVERSA O ESCALIND:\n');
disp(WI);
fprintf(1,'\n TRANSFORMADA INVERSA:\n');
disp(xr);

%% PSNR
fprintf('\n PSNR= %f dB\n', calculoPSNR(Ycomp, xr));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                              %%
%% COMPRESION DE VIDEO                                          %%
%% Grado en Ingeniera de Tecnologas de Telecomunicacin       %%
%% Gustavo A. Ruiz Robredo                                      %%
%% ruizrg@unican.es                                             %%
%%                                                              %%
%% uu   uu  cccccc   UNIVERSIDAD DE CANTABRIA                   %%
%% uu   uu  cccccc   Dpto. INGENIERIA INFORMATICA Y ELECTRONICA %%
%% uu   uu  cc       FACULTAD DE CIENCIAS                       %%
%%  uuuuu   cccccc   Avda. Los Castros s/n                      %%
%%   uuu    cccccc   39005-SANTANDER (SPAIN)                    %%
%%                                                              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

