function psnr = calculoPSNR(imgA, imgB)
%
% Calcula le PSNR der dos imagenes
%
% Entradas:
%   imgA : primera imagen
%   imgB : segunda imagen
%
%   Ambas imagenes deben tener la misma dimensin
%
% Salida:
%   psnr -> Peak Signal to noise rate
%

[MA NA]=size(imgA);
[MB NB]=size(imgB);

% Comprobacion de la correcta dimension
if ( (MA ~= MA) || (NB ~= NB) )
    fprintf(1,'\n DIM imagen A (%d x %d) no coincide con el de imagen B (%d x %d).\n',MA, NA, MB, NB);
    return;
end

% Rms ^2
Error2 = (double(imgA)-double(imgB)).^2;

mse=(sum(sum(sum(Error2)))/(MA*NA));

% PSNR
if ( Error2 == 0)
    psnr=inf;
else
    psnr = 10*log10(255^2 / mse);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                              %%
%% COMPRESION DE VIDEO                                          %%
%% Grado en Ingeniera de Tecnologas de Telecomunicacin       %%
%% Gustavo A. Ruiz Robredo                                      %%
%% ruizrg@unican.es                                             %%
%%                                                              %%
%% uu   uu  cccccc   UNIVERSIDAD DE CANTABRIA                   %%
%% uu   uu  cccccc   Dpto. INGENIERIA INFORMATICA Y ELECTRONICA %%
%% uu   uu  cc       FACULTAD DE CIENCIAS                       %%
%%  uuuuu   cccccc   Avda. Los Castros s/n                      %%
%%   uuu    cccccc   39005-SANTANDER (SPAIN)                    %%
%%                                                              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
