function [Y, U, V]=leerVideoYUV_420(nombreImagen, high, width, numFrame);
% Funcion que lee una un video en formato YUV 4:2:0 8 bits
% Argumentos de entrada:
%    nombreImagen: Nombre de la imagen
%    high: Altura
%    width: Anchura
%    numFrame: Nmero de frames
%
% Argumentos de salida:
%    Y, U, V -> matrices tridimensionales conteniendo la luma (Y) y dos
%    cromas (U, V) de forma que
%    Y, U, V (NUM; width; high)
%
% AASe abre el video
fid = fopen(nombreImagen);
if( fid == -1)
    fprintf(1,'\n ERROR: Imposible de abrir el fichero de entrada %s.\n',nombreImagen);
    return;
end

% Se reserva la memoria
Y=uint8(zeros(high, width,numFrame ));
U=uint8(zeros(floor(high/2), floor(width/2),numFrame ));
V=uint8(zeros(floor(high/2), floor(width/2), numFrame ));

% Se lee el video y se almacena en el array arrayImagen
arrayImagen = fread(fid,'uint8=>uint8');

% Se deinen los lmites
lmt1=high*width;
lmt2=lmt1+floor(high/2)*floor(width/2);
lmt3=lmt2+floor(high/2)*floor(width/2);

for fr=1:numFrame % numero de frames
    for i=1:high
        for j=1:width
            Y(i,j,fr)=arrayImagen((fr-1)*lmt3+(i-1)*width+j);
        end
    end
    for i=1:floor(high/2)
        for j=1:floor(width/2)
            U(i,j,fr)=arrayImagen((fr-1)*lmt3+lmt1+(i-1)*floor(width/2)+j);
            V(i,j,fr)=arrayImagen((fr-1)*lmt3+lmt2+(i-1)*floor(width/2)+j);
        end
    end
end
status = fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                              %%
%% COMPRESION DE VIDEO                                          %%
%% Grado en Ingeniera de Tecnologas de Telecomunicacin       %%
%% Gustavo A. Ruiz Robredo                                      %%
%% ruizrg@unican.es                                             %%
%%                                                              %%
%% uu   uu  cccccc   UNIVERSIDAD DE CANTABRIA                   %%
%% uu   uu  cccccc   Dpto. INGENIERIA INFORMATICA Y ELECTRONICA %%
%% uu   uu  cc       FACULTAD DE CIENCIAS                       %%
%%  uuuuu   cccccc   Avda. Los Castros s/n                      %%
%%   uuu    cccccc   39005-SANTANDER (SPAIN)                    %%
%%                                                              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%